#ifndef FGGL_GFX_ECS_CAM_H
#define FGGL_GFX_ECS_CAM_H

#include <fggl/ecs2/ecs.hpp>
#include <fggl/math/types.hpp>
#include <fggl/math/ecs.hpp>

#include <fggl/gfx/input.hpp>

namespace fggl::components {

	struct Camera {
		math::vec3 target = math::vec3(0.0f, 0.0f, 0.0f);
		float aspectRatio = 1280.0f / 720.0f;
		float fov = glm::radians(45.0f);
		float nearPlane = 0.1f;
		float farPlane = 100.0f;
	};

	class CameraMod {
		public:
			inline CameraMod(ecs2::impl::world& world) {
				world.module<CameraMod>("cam");
				world.component<Camera>("camera");
			}
	};
};

namespace fggl::systems {

	void CameraArcball(fggl::ecs2::impl::iter& it, fggl::components::Transform* t, fggl::components::Camera* c);
	void CameraFree(fggl::ecs2::impl::iter& it, fggl::components::Transform* t, fggl::components::Camera* c);
	void CameraEdgeScroll(fggl::gfx::Input& input, fggl::components::Transform& t, fggl::components::Camera& c);
	void ProcessCameraInput(fggl::ecs2::impl::iter& it, fggl::components::Transform* t, fggl::components::Camera* c);

	class CameraInput {
		public:
			inline CameraInput(ecs2::impl::world& world) {
				world.module<CameraInput>("CamInput");

				world.system<fggl::components::Transform, fggl::components::Camera>()
					.iter( ProcessCameraInput );
			}
	};

}

#endif
