#ifndef FGGL_GFX_ECS_H
#define FGGL_GFX_ECS_H

#include "window.hpp"
#include <fggl/ecs2/ecs.hpp>
#include <fggl/math/types.hpp>

#include <fggl/math/ecs.hpp>
#include <GL/gl.h>

namespace fggl::components {

	struct GfxToken {
		GLuint vao;
		GLuint buffs[2];
		GLuint idxOffset;
		GLuint idxSize;
		GLuint pipeline;
	};

	struct GfxMat {
		math::mat4 model;
		math::mat4 view;
		math::mat4 proj;
	};

	// FIXME imperfect abstraction - needs fixing
	struct GfxHacks {
		fggl::gfx::Window* window;
	};

	class Gfx {
		public:
			inline Gfx(ecs2::impl::world& world) {
				world.module<Gfx>("gfx");
				world.component<GfxToken>("token");
				world.component<GfxMat>("mat");
			}
	};
};

namespace fggl::systems {


	void fglPopulateVertex(flecs::iter& it, const fggl::components::Transform* t, fggl::components::GfxMat* mats);
	void fglDrawMeshes(flecs::iter& it);

	class OpenGL {
		public:
			inline OpenGL(ecs2::impl::world& world) {
				world.module<OpenGL>("ogl");

				world.system<const fggl::components::Transform, fggl::components::GfxMat>("VertPrep")
					.kind(flecs::OnStore)
					.iter(fglPopulateVertex);

				world.system<>("DrawSys", "gfx.mat,SHARED:gfx.token")
					.kind(flecs::OnStore)
					.iter(fglDrawMeshes);
			}
	};

}

#endif
